import unittest

import numpy as np

from conf.testing import czifile, output_path
from model_server.image import CziImageFileAccessor, DataShapeError, InMemoryDataAccessor, write_accessor_data_to_file

class TestCziImageFileAccess(unittest.TestCase):
    def setUp(self) -> None:
        pass

    def test_czifile_is_correct_shape(self):
        cf = CziImageFileAccessor(czifile['path'])
        self.assertEqual(cf.shape_dict['Y'], czifile['h'])
        self.assertEqual(cf.shape_dict['X'], czifile['w'])
        self.assertEqual(cf.chroma, czifile['c'])
        self.assertFalse(cf.is_3d())
        self.assertEqual(len(cf.data.shape), 4)
        self.assertEqual(cf.shape[0], czifile['h'])
        self.assertEqual(cf.shape[1], czifile['w'])

    def test_write_single_channel_tif(self):
        ch = 4
        cf = CziImageFileAccessor(czifile['path'])
        mono = cf.get_one_channel_data(ch)
        self.assertTrue(
            write_accessor_data_to_file(
                output_path / f'{cf.fpath.stem}_ch{ch}.tif',
                mono
            )
        )
        self.assertEqual(cf.data.shape[0:2], mono.data.shape[0:2])
        self.assertEqual(cf.data.shape[3], mono.data.shape[2])

    def test_conform_data_shorter_than_xycz(self):
        data = np.random.rand(256, 512, 1)
        acc = InMemoryDataAccessor(data)
        self.assertEqual(
            InMemoryDataAccessor.conform_data(data).shape,
            (256, 512, 1, 1)
        )
        self.assertEqual(
            acc.shape_dict,
            {'X': 256, 'Y': 512, 'C': 1, 'Z': 1}
        )

    def test_conform_data_longer_than_xycz(self):
        data = np.random.rand(256, 512, 12, 8, 3)
        with self.assertRaises(DataShapeError):
            acc = InMemoryDataAccessor(data)